/* 
   amfe.cmd 
   version 1.40 (20010506 )
   
   History:
    - 19980728
    - 19971119

   A simple commandline frontend for PMMSend
   (which ships with PMMail from SouthSoft,Inc)

     written by
     Ŀ
         Alexander Mai                          
         st002279@hrzpub.tu-darmstadt.de        
         Alexander.Mai@physik.tu-darmstadt.de   
     

*/

/*
   Todo:
        - check for matching directory structure
*/


program = 'AMFE'
version = 1.40

/* defining some constants */
True    = 1
False   = 0
_beep   = D2C(7)
_CRLF   = D2C(13)''D2C(10)

options = ''

/* Fetching and parsing the commandline arguments */
Parse Arg arguments
call PCLM

/* We need some SysUtil()-functions */
if RxFuncQuery('SysLoadFuncs') = 1 then
  if RxFuncAdd('SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs') <> 0 then
    do
    call OutError 'Sorry, your system seems to lack the REXX-Util-library !'
    call OutError program' can not run on this system :-('
    SIGNAL SCHLUSS
    end
  else
    call SysLoadFuncs

if Translate(Left(param.1,8)) = 'SETDEFAULT=' then
  /* Explicit setting of default account */
  do
  if CheckSetup(True) = False then
    do
    call OutError 'There is no valid configuration for 'program
    SIGNAL SCHLUSS
    end
  defacct = SubStr(param.1,9)
  rc = SysIni(, 'AMFE', 'DefAcct', defacct)
  if rc = '' then
    call OutNormal 'Default account: ' SysIni(, 'AMFE', 'DefAcct')
  else
    call OutError 'Failed to set default account'
  SIGNAL SCHLUSS
  end

/* Initialize some variables */
acct        = ''
acct_dir    = ''
subject     = ''
to_address  = ''
try_default = False
/* (too ?) simple test whether we should read direct from stdin
   instead of using interactive mode */
if Chars() > 0 then
  use_stdin = True
else
  use_stdin = False

/* We try to accept as much information as possible from
   the commandline */
do i = 1 to param.0
  curpar = param.i
  if Left(curpar,1) = '/' then
    options = options''Translate(curpar)
  else
    /* curpar is no option */
    do
    if (param.0 < 3) then
      do
      if to_address = '' then
        to_address = curpar
      else
        subject = curpar
      try_default = True
      end
    else
      /* >=3 parameters */
      do
      if acct = '' then 
        acct = curpar
      else
        if to_address = '' then
          to_address = curpar
        else
          subject = curpar
      end
    end
  end

if options <> '' then
  select
    when Pos('D', options) > 0 then
      nop
    when Pos('H', options) > 0 then
      SIGNAL HelpnQuit
    when Pos('N', options) > 0 then
      nop
    when Pos('S', options) > 0 then
      nop
    when Pos('U', options) > 0 then
      do
      call UnInstall
      SIGNAL SCHLUSS
      end
    when Pos('V', options) > 0 then
      do
      call ShowVersion
      SIGNAL SCHLUSS
      end
    otherwise
      SIGNAL HelpnQuit
    end /* select */

/* Check whether we find a proper configuration */
if CheckSetup(True) = False then
  do
  call OutError 'There is no valid configuration for 'program
  SIGNAL SCHLUSS
  end
rc = CheckDefaults(False)
if try_default then
  /* Try to use default account */
  do
  if rc=False then
    do
    say 'No default account found'
    SIGNAL SCHLUSS
    end
  else
    do
    rc = CheckAccount( def_acct, True )
    if rc = False then
      do
      call OutError program' is not configured properly'
      SIGNAL SCHLUSS
      end
    else
      acct = def_acct 
    end
  end
else
  do
  if Strip(acct) = '' then
    do
    rc = CheckAccount( def_acct, False )
    if rc = False then
      def_acct = ''
    end
  else
    do
    rc = CheckAccount( acct, False )
    if rc = False then
      do
      call OutError program' is not configured properly'
      SIGNAL SCHLUSS
      end
    end
  end
  
/* Create temporary file to keep the message body */
tmppath = Value('TMP', , 'OS2ENVIRONMENT')
if tmppath = '' then
  tmppath = Value('TEMP', , 'OS2ENVIRONMENT')
if tmppath = '' then
  tmppath = '.'
tmpfile = SysTempFileName(tmppath'\APSW.???')
if tmpfile = '' then
  SIGNAL SCHLUSS
body = ''
eom  = False
if use_stdin = False then
  do
  /* Get all information in dialog with user */
  if acct = '' then
    do
    if def_acct = '' then
      say 'Account ("EXIT" to quit):'
    else
      say 'Account ("EXIT" to quit, RETURN to use default account):'
    Parse Pull acct
    acct = Strip(acct)
    if acct = '' then
      do
      if def_acct = '' then
        do
        call OutError 'No account given'
        SIGNAL SCHLUSS
        end
      else
        /* use default account */
        acct = def_acct
      end
    else
      /* acct <> '' */
      do
      if Translate(acct) = 'EXIT' then
        SIGNAL SCHLUSS
      else
        if CheckAccount( acct, True ) = False then
          SIGNAL SCHLUSS
      end
    end
  if to_address = '' then
    do
    Say 'To:'
    Parse Pull to_address
    if CheckAddress(to_address) = False then
      do
      call OutError 'Invalid email address'
      SIGNAL SCHLUSS
      end
    end
  if subject = '' then
    do
    Say 'Subject:'
    Parse Pull subject
    end
  Say 'Message:'
  do while \eom
    Parse Pull line
    if line = '.' then
      eom = True
    else
      body = body''line''_CRLF
    end /* do while */
  end
else
  /* just read from stdin */
  do
  do while Lines() > 0 
    body = body''LineIn()''_CRLF
    end
  do while Chars() > 0 
    body = body''CharIn()
    end
  end
  
if IsOptionSet('N') = False then
  do
  /* Try to add default signature */
  if signature_file <> '' then
    do
    call LineIn signature_file,1,0
    signature_length = Chars(signature_file)
    signature = CharIn(signature_file,1,signature_length)
    call LineOut signature_file
    end
  end
 
/* Create temporary file that PMMSend can read */
call LineOut tmpfile,,1
call CharOut tmpfile, body
call CharOut tmpfile, signature
call LineOut tmpfile
/* Wrap subject and address in quotation marks */
if subject = '' then
  subject = '""'
else
  subject = '"'subject'"'
to_address = '"'to_address'"'
if IsOptionSet('D') = True then
  _verbose = ''
else
  _verbose='@'
/* Build the commandline */
pmms_cmd = _verbose'pmmsend.exe /m 'tmpfile' 'to_address' 'subject' 'acct_dir

/* Change to PMMSend's directory and execute <pmms_cmd> */
curdir = Directory()
call Directory(pmm_path)
Address CMD pmms_cmd' 1>nul'
call Directory(curdir)


/* Check for returncode to see whether we had success
   though I don't know whether this does actually work ...     */
if rc = 0 then
  call OutNormal 'Message queued/sent !'
else
  do
  call OutNormal 'Sending was not successful'
  call OutNormal 'RC(PMMSend):'rc
  end
/* Not much work left: clean up, restore working directory, ... */
call SysFileDelete(tmpfile)

SCHLUSS:
/* all program branches should end here */
exit


/* ######################################################################### */


/* All used inline subroutines and functions follow in
   no specific order */

OutNormal:
/* Normal output. Omitted when 'S' is set */
Parse Arg on_param
if IsOptionSet('S') = False then
  say on_param
return


OutError:
/* Error output. Never omitted ? Should go to stderr ?? */
Parse Arg on_param
say on_param
return


CheckAddress:
/* A difficult job, we try to do a reasonable minimum here
  (for multiple accounts on the localhost this might even be
   too restrictive) */
Parse Arg adr
first   = Pos('@',adr)
last    = LastPos('@',adr)
letters = Length(adr)
/* We check for
  - included blanks
  - only one "@" and not at the beginning
*/
if (Words(adr) = 1) & (first > 0) & (first = last) then
  return True
else
  return False


PCLM:
/* Parse command line module */
/* expects the whole string in arguments */
_SINGLE = D2C(39)
_DOUBLE = D2C(34)
min = 1
count = 0
arguments=Strip(arguments)
do until min = 0
  single = Pos("'", arguments)
  double = Pos('"', arguments)
  min = minimum(single' 'double)
  if min = 0 then
    do
    do while arguments <> ''
      Parse Var arguments wort arguments
      count= count + 1
      param.count = wort
      end
    end
  else
    do
    if single = min then
      do
      Parse Var arguments head (_SINGLE) body (_SINGLE) rest
      do while head <> ''
        Parse Var head wort head
        count= count + 1
        param.count = wort
        end
      if Length(body) > 0 then
        do
        count = count + 1
        param.count = body
        arguments = rest
        end
      end
    else
      do
      Parse Var arguments head (_DOUBLE) body (_DOUBLE) rest
      do while head <> ''
        Parse Var head wort head
        count= count + 1
        param.count = wort
        end
      if Length(body) > 0 then
        do
        count = count + 1
        param.count = body
        arguments = rest
        end
      end
    end
param.0 = count
return


CheckDefaults:
/* Get all default data for AMFE (currently only from OS2.INI)
   and sets 'def_acct' */
Parse Arg cd_verbose
def_acct = SysIni(, 'AMFE', 'DefAcct')
if (def_acct = 'ERROR:') | (def_acct = '') then
  do
  if cd_verbose = True then
    call OutError 'No default account found'
  return False
  end
else
  return True


CheckSetup:
/* Check whether PMMail and PMMSend are in path and
   whether there's the matching directory structure */
Parse Arg cs_verbose
cs_error = 0
/* PMMSend.exe should be in PATH namely in the same directory
   as PMMail.exe */
InstallRoot = PMMailRoot()
pmms_exe = SysSearchPath('PATH', 'pmmsend.exe')
if pmms_exe = '' then
  do
  if cs_verbose = True then
     call OutError 'PMMSend not in path'
  try=InstallRoot'\PMMSend.exe'
  if FileExists(try) then
     do
     pmms_exe = try
     say "Using "pmms_exe
     end
  else
     do
     cs_error = cs_error + 1
     end
  end
pmmail_exe = SysSearchPath('PATH', 'PMMail.exe')
if pmmail_exe = '' then
  do
  if cs_verbose = True then
    call OutError 'PMMail not in path'
  try=InstallRoot'\PMMail.exe'
  if FileExists(try) then
     do
     pmmail_exe = try
     say "Using "pmmail_exe
     end
  else
     do
     cs_error = cs_error + 1
     end
  end

pmm_path = FileSpec('drive', pmms_exe)''FileSpec('path', pmms_exe)
pmm_path = Strip(pmm_path, 'T', '\')
if cs_error = 0 then
  return True
else
  return False
  

CheckAccount:
/* Check whether an account specifyed by it's name exists */
Parse Arg ca_acct, ca_verbose
_SEP = X2C('DE')
/* Search all account directories for the given account name */
/*say 'searching:'ca_acct' in 'pmm_path*/
rc = SysFileTree(pmm_path'\*.ACT', 'accdirs', 'DO')
file = 'ACCT.CFG'
match = 0
do i = 1 to accdirs.0
  if Stream(accdirs.i'\'file, 'C', 'QUERY EXISTS') <> '' then
    do
    value = SysIni(accdirs.i'\'file, 'ACCT', 'ACCTNAME')
    if (Translate(DelStr(value,Length(value))) = Translate(ca_acct)) then
      match = i
    end
  end
if match = 0 then
  do
  if ca_verbose = True then
    call OutError 'Invalid account'
  acct_dir = ''
  signature_file = ''
  return False
  end
else
  do
  acct_dir = accdirs.match
  sig_list = acct_dir'\sigs\sigs.lst'
  sig_val = 0
  call LineIn sig_list,1,0
  do while (Lines(sig_list) > 0) & (sig_val = 0)
    sig_line =Strip(LineIn(sig_list))
    Parse Var sig_line sig_name (_SEP) sig_file (_SEP) sig_val (_SEP)
    end
  call LineOut sig_list
  if sig_line = '' then
    signature_file = ''
  else
    signature_file = Stream(pmm_path'\'sig_file, 'C', 'QUERY EXISTS')
  return True
  end

  
PMMailRoot: Procedure
/* ask in the INI where PMMail is installed */
value = SysIni(, 'PMMail/2', 'InstallRoot')
/* experience shows that those might be zero-terminated string. Skip them */
value=Strip(value, 'T', d2c(0))
return value


IsOptionSet:
/* Just an elegant abbreviation */
Parse Arg o_param
if Pos(o_param,options) > 0 then
  return True
else
  return False


Minimum:
/* syntax: Minimum( a b ... )           */
/* returns the smallest number > 0 or 0 */
m_min = 0
Parse Arg m_param
do until m_param = ''
  Parse Var m_param m_cand m_param
  if (m_cand > 0) then
    do
    if m_min = 0 then
      m_min = m_cand
    else
      if m_cand < m_min then
        m_min = m_cand
    end
  end
return m_min


UnInstall:
/* Currently just remove entries from OS2.INI */
Parse Arg ui_param
rc = SysIni(, 'AMFE', 'DELETE:')
if (rc <> '') | (rc = 'ERROR:') then
  call OutNormal 'Error while uninstalling 'program
else
  call OutNormal program' uninstalled successfully'
return


HelpnQuit:
/* Show Help screen and Quit */
say program" {<options>} {<account>} <to_address> {<subject>}"
say "  or"
say program" setdefault=<default account>"
say
say "Use "program" to send email from commandline using PMMSend as an interface"
say "to PMMail. PMMail and PMMsend have to be installed and available"
say "in path. "program" puts the email in PMMail's outbox so that they will be"
say "sent the next time PMMail sends messages."
say
say "If a default account has been set you can just specify <to_address> and"
say "let "program" read from stdin. Otherwise "program" will ask you for the missing"
say "information. Note that you can just edit the actual line of the message"
say "and a line containing just a single period '.' indicates the end."
say
say "Valid options are:"
say "  /d  show some debug info"
say "  /h  show this help screen"
say "  /n  do not use the default signature"
say "  /s  run silently (less verbose)"
say "  /u  uninstall (remove entries from OS2.INI)"
say "  /v  show some program infos and the default account"
SIGNAL SCHLUSS


ShowVersion:
/* Display a short status */
say program' version: 'version
say 'Written by Alexander Mai'
Parse Version rexxrelease
say 'REXX-Interpreter: 'rexxrelease
if CheckSetup(True) = False then
  do
  say 'There is no valid configuration for 'program
  SIGNAL SCHLUSS
  end
rc = CheckDefaults(False)
if rc = True then
  do
  rc = CheckAccount(def_acct, False)
  if rc = False then
    say 'Can not locate account 'def_acct
  else
    do
    say 'Default account:   'def_acct
    say 'Default signature: 'signature_file
    if signature_file <> '' then
      do
      call LineIn signature_file,1,0
      do while Lines(signature_file) > 0 
        say LineIn(signature_file)
      end
      do while Chars(signature_file) > 0 
        say CharIn(signature_file)
      end
      call LineOut signature_file
      end /* if signature_file */
    end
  end
else
  say 'No default account found.'
return


FileExists: Procedure
/*  Returns 1 if a given File exists */
Parse Arg is_file
rs = Stream(is_file, 'C', 'QUERY EXISTS')
if rs == "" then
  return 0
else
  return 1
